/**
 * Created by jmoutard on 19/07/16.
 */

'use strict';
/*
 * Server Storage Service give data from the server if logged in and is capable of giving errors.
 * Errors are :
 *          - 0; Not authenticated (not implemented yet),
 *          - 1; Error when adding dashboard (There is no 'replace dashboard', this is the same error when adding a new dashboard or overriding an already existent one),
 *          - 2; Dashboard not found (same error for one and for all dashboard),
 *          - 3; Can't remove the dashboard.
 *
 * For convenience error code from 0 to 9 belongs to server storage errors.
 */
angular.module('adf.electre')
    .service('serverStorageService', [
        "$http", "$q", "storageErrorProvider",
        function ($http, $q, storageErrorProvider) {
            /**
             * @description Return all stored dashboard from the server.
             * @returns {Promise}
             */
            function getAll () {
                var deferred = $q.defer();
                $http.get('/getAll')
                    .success(function(data){
                        deferred.resolve(data.dashboards);
                    })
                    .error(function(){
                        deferred.reject(new storageErrorProvider(2, "No dashboard found."));
                    });
                return deferred.promise;
            }

            /**
             * @description Return a specific stored dashboard from the server.
             * @param {string} id
             * @returns {Promise}
             */
            function get (id) {
                var deferred = $q.defer();
                $http.get('/get/' + id)
                    .success(function(data){
                        deferred.resolve(data);
                    })
                    .error(function(){
                        deferred.reject(new storageErrorProvider(2, "No dashboard found."));
                    });
                return deferred.promise;
            }

            /**
             * @description Add a dashboard onto the server.
             * @param {string} id
             * @param {object} data
             * @returns {Promise}
             */
            function add (id, data) {
                var deferred = $q.defer();
                $http.post('/set/' + id, data)
                    .success(function(data){
                        deferred.resolve();
                    })
                    .error(function(){
                        deferred.reject(new storageErrorProvider(1, "Can't add the dashboard."));
                    });
                return deferred.promise;
            }

            /**
             * @description Remove a dashboard onto the server.
             * @param {string} id
             * @returns {Promise}
             */
            function remove (id) {
                var deferred = $q.defer();
                $http.delete('/delete/' + id)
                    .success(function(data){
                        deferred.resolve(data);
                    })
                    .error(function(){
                        deferred.reject(new storageErrorProvider(3, "Can't remove the dashboard."));
                    });
                return deferred.promise;
            }

            return { getAll: getAll, get: get, add: add, remove: remove};
        }
    ]);